﻿
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports FarseerPhysics.Dynamics
Imports Microsoft.Xna.Framework.Graphics
Imports Microsoft.Xna.Framework
Imports FarseerPhysics.Factories
Imports FarseerPhysics.Common
Imports FarseerPhysics.Collision.Shapes


Public Class DrawablePhysicsObject
    ' Because Farseer uses 1 unit = 1 meter we need to convert
    ' between pixel coordinates and physics coordinates.
    ' I've chosen to use the rule that 100 pixels is one meter.
    ' We have to take care to convert between these two 
    ' coordinate-sets wherever we mix them!

    Public Const unitToPixel As Single = 100.0F
    Public Const pixelToUnit As Single = 1 / unitToPixel

    Public body As Body
    Public Property Position() As Vector2
        Get
            Return body.Position * unitToPixel
        End Get
        Set(value As Vector2)
            body.Position = value * pixelToUnit
        End Set
    End Property

    Public texture As Texture2D

    Private m_size As Vector2
    Public Property Size() As Vector2
        Get
            Return m_size * unitToPixel
        End Get
        Set(value As Vector2)
            m_size = value * pixelToUnit
        End Set
    End Property

    ''' <param name="world">The farseer simulation this object should be part of</param>
    ''' <param name="texture">The image that will be drawn at the place of the body</param>
    ''' <param name="size">The size in pixels</param>
    ''' <param name="mass">The mass in kilograms</param>
    Public Sub New(world As World, texture As Texture2D, size As Vector2, mass As Single)
        body = BodyFactory.CreateRectangle(world, size.X * pixelToUnit, size.Y * pixelToUnit, 1)
        'body.Restitution = 0.6 'rebond
        'body.Friction = 0.0
        '
        body.BodyType = BodyType.Dynamic

        Me.Size = size
        Me.texture = texture
    End Sub

    Public Sub Draw(spriteBatch As SpriteBatch)
        Dim scale As New Vector2(Size.X / CSng(texture.Width), Size.Y / CSng(texture.Height))
        'the texture has no link whatsoever with our world - it is there only to draw
        spriteBatch.Draw(texture, Position, Nothing, Color.White, body.Rotation, New Vector2(texture.Width / 2.0F, texture.Height / 2.0F), scale, SpriteEffects.None, 0)
    End Sub

    
End Class
