﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics
Imports Microsoft.Xna.Framework.Audio
Imports Microsoft.Xna.Framework.Content
Imports Microsoft.Xna.Framework.Media
Imports Microsoft.Xna.Framework.Input

Imports FarseerPhysics
Imports FarseerPhysics.Dynamics
Imports FarseerPhysics.Collision
Imports FarseerPhysics.Factories




Public Class Game
    Inherits Microsoft.Xna.Framework.Game
    Dim graphics As GraphicsDeviceManager
    Dim SpriteBatch As SpriteBatch

    Private world As World
    Private crateList As List(Of DrawablePhysicsObject)
    Private floor As DrawablePhysicsObject

    Private prevKeyboardState As KeyboardState
    Private random As Random
    Dim i As Integer
    Dim myfont As SpriteFont


    Public Sub New()
        graphics = New GraphicsDeviceManager(Me)
        Window.Title = "Test"
        graphics.PreferMultiSampling = True
    End Sub

    Protected Overrides Sub Initialize()
        'TODO: Add your initialization logic here'

        MyBase.Initialize()


    End Sub
    Protected Overrides Sub LoadContent()
        SpriteBatch = New SpriteBatch(GraphicsDevice)
        myfont = Content.Load(Of SpriteFont)("xnb\myfont")
        random = New Random()

        ' World is the most important farseer object. It's where
        ' all the objects should be registered and it handles the 
        ' entire simulation via the step function.
        ' Here we instantiate it with earth like gravity
        world = New World(New Vector2(0, 9.81F))

        floor = New DrawablePhysicsObject(world, Content.Load(Of Texture2D)("xnb\Floor"), New Vector2(GraphicsDevice.Viewport.Width, 100.0F), 1000)
        floor.Position = New Vector2(GraphicsDevice.Viewport.Width / 2.0F, GraphicsDevice.Viewport.Height - 50)
        floor.body.BodyType = BodyType.[Static]

        crateList = New List(Of DrawablePhysicsObject)()

        prevKeyboardState = Keyboard.GetState()

        MyBase.LoadContent()

    End Sub
    Protected Overrides Sub UnloadContent()
        MyBase.UnloadContent()
        'TODO: Unload any non ContentManager content here'
    End Sub

    ''' <summary>
    ''' Spawn a crate at a random position at the top of the screen
    ''' </summary>
    Private Sub SpawnCrate()
        Dim crate As DrawablePhysicsObject
        crate = New DrawablePhysicsObject(world, Content.Load(Of Texture2D)("xnb\crate"), New Vector2(50.0F, 50.0F), 0.1F)
        crate.Position = New Vector2(random.[Next](50, GraphicsDevice.Viewport.Width - 50), 1)
        Debug.WriteLine("creating: " & crate.body.BodyId)
        'collision handler
        AddHandler crate.body.OnCollision, AddressOf body_Oncollision

        crateList.Add(crate)
    End Sub

    Protected Overrides Sub Update(ByVal gameTime As Microsoft.Xna.Framework.GameTime)

        Dim keyboardState As KeyboardState = Keyboard.GetState()
        If keyboardState.IsKeyDown(Keys.Space) AndAlso Not prevKeyboardState.IsKeyDown(Keys.Space) Then
            SpawnCrate()
        End If

        prevKeyboardState = keyboardState

        world.Step(CSng(gameTime.ElapsedGameTime.TotalSeconds))


        MyBase.Update(gameTime)

    End Sub


    Protected Overrides Sub Draw(ByVal gameTime As Microsoft.Xna.Framework.GameTime)
        GraphicsDevice.Clear(Color.CornflowerBlue)

        SpriteBatch.Begin(SpriteSortMode.Immediate, BlendState.Opaque)

        For Each crate As DrawablePhysicsObject In crateList
            crate.Draw(SpriteBatch)
        Next

        floor.Draw(SpriteBatch)

        SpriteBatch.DrawString(myfont, "collisions: " & i.ToString, New Vector2(20, 20), Color.White)

        SpriteBatch.End()

        MyBase.Draw(gameTime)
    End Sub

    Private Function body_Oncollision(fixtureA As Fixture, fixtureB As Fixture, contact As Contacts.Contact) As Boolean
        Debug.WriteLine("A:" & fixtureA.Body.BodyId & " colliding with B:" & fixtureB.Body.BodyId)
        i = i + 1
        Return True 'false will let the object go thru
    End Function

End Class

